package com.jiuqi.office.excel;

package .tmp;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.apache.poi.hssf.util.HSSFColor;

public class HSSFHelper {
  private static final int[] WIN32_COLORS = new int[] { 
      13158600, 13743257, 14405055, 
      15790320, 16777215, 6579300, 
      11842740, 16578548, 
      11250603, 16750899, 
      16777215, 15790320, 
      10526880, 8421504, 5525059, 
      16777215, 6908265, 14935011, 
      14811135, 13395456, 
      15389113, 
      15918295, 16750899, 15790320 };
  
  public static int pixelToWidth(int value) {
    if (value <= 0)
      return 0; 
    if (value == 1)
      return 19; 
    int width = 19;
    value--;
    width += value / 7 * 256;
    int delta = value % 7;
    if (delta > 0) {
      width += delta / 2 * 73;
      if (delta % 2 == 1)
        width += 37; 
    } 
    return width;
  }
  
  public static int widthToPixel(short value) {
    return (short)(value / 32);
  }
  
  public static short pixelToHeight(int value) {
    if (value <= 0)
      return 0; 
    int height = 7 + (value - 1) * 15;
    return (short)height;
  }
  
  public static int heightToPixel(short value) {
    return value / 15;
  }
  
  public static HSSFColor getApproximateColor(int rgbColor) {
    Integer key = new Integer(rgbColor);
    HSSFColor retColor = (HSSFColor)colorFinder.get(key);
    if (retColor == null) {
      retColor = findApproximateColor(rgbColor);
      if (retColor != null)
        colorFinder.put(key, retColor); 
    } 
    return retColor;
  }
  
  public static int getRGBColor(HSSFColor hssfColor) {
    short[] rgb = hssfColor.getTriplet();
    return rgb[0] << 16 | rgb[1] << 8 | rgb[2];
  }
  
  public static int getRGBColor(short index) {
    HSSFColor hssfColor = (HSSFColor)HSSFColor.getIndexHash().get(new Integer(index));
    if (hssfColor != null)
      return getRGBColor(hssfColor); 
    return -1;
  }
  
  public static HSSFColor getApproximateTextColor(int rgbColor) {
    if (rgbColor == -16777211)
      rgbColor = 0; 
    return getApproximateColor(rgbColor);
  }
  
  public static HSSFColor getApproximateEdgeColor(int rgbColor) {
    if (rgbColor == -16777201)
      rgbColor = 8421504; 
    return getApproximateColor(rgbColor);
  }
  
  private static void buildColorList() {
    synchronized (colorElements) {
      if (colorElements.size() == 0) {
        Map colorMaps = null;
        colorMaps = HSSFColor.getIndexHash();
        Collection colorList = colorMaps.values();
        Iterator<HSSFColor> colorItr = colorList.iterator();
        while (colorItr.hasNext()) {
          ColorElement element = new ColorElement();
          element.color = colorItr.next();
          short[] rgbVals = element.color.getTriplet();
          element.rgb = rgbToColor(rgbVals[0], rgbVals[1], rgbVals[2]);
          element.hsb = color2HSB(element.rgb);
          colorElements.add(element);
        } 
      } 
    } 
  }
  
  private static HSSFColor findApproximateColor(int rgbColor) {
    buildColorList();
    HSB Hsb = color2HSB(rgbColor);
    double L = Math.sqrt(20000.0D);
    ColorElement flagElement = null;
    for (int i = 0; i < colorElements.size(); i++) {
      ColorElement elem = colorElements.get(i);
      if (rgbColor == elem.rgb)
        return elem.color; 
      double Q = Hsb.h - elem.hsb.h;
      if (Q > 180.0D)
        Q = 360.0D - Q; 
      double E = Math.sqrt(Math.pow(elem.hsb.s - Hsb.s, 2.0D) + Math.pow(elem.hsb.b - Hsb.b * Math.cos(Q * Math.PI / 180.0D), 2.0D) + Math.pow(Hsb.b * Math.sin(Q * Math.PI / 180.0D), 2.0D));
      if (L > E) {
        L = E;
        flagElement = elem;
      } 
    } 
    if (flagElement == null)
      return null; 
    return flagElement.color;
  }
  
  private static List colorElements = new ArrayList();
  
  private static Map colorFinder = new Hashtable<Object, Object>();
  
  private static int rgbToColor(int r, int g, int b) {
    r <<= 16;
    g <<= 8;
    return r | g | b;
  }
  
  private static HSB color2HSB(int value) {
    HSB hsb = new HSB();
    RGB rgb = colorToRGB(value);
    double M0 = maxColorValue(rgb) / 2.55D;
    hsb.setDefault();
    hsb.b = M0;
    if (hsb.b == 0.0D) {
      hsb.s = 0.0D;
    } else {
      double M1 = minColorValue(rgb) / 2.55D;
      hsb.s = (M0 - M1) * 100.0D / M0;
    } 
    hsb.h = RGBToHue(rgb);
    return hsb;
  }
  
  private static RGB colorToRGB(int value) {
    if (value < 0)
      value = WIN32_COLORS[value & 0xFF]; 
    int r = (value & 0xFF0000) >> 16;
    int g = (value & 0xFF00) >> 8;
    int b = value & 0xFF;
    return new RGB(r, g, b);
  }
  
  private static double RGBToHue(RGB rgb) {
    double H;
    int M0 = minColorValue(rgb);
    int M2 = maxColorValue(rgb);
    if (M0 == M2)
      return 0.0D; 
    int M1 = centerColorValue(rgb);
    if (M2 == M1) {
      if (rgb.r == rgb.g)
        return 60.0D; 
      if (rgb.g == rgb.b)
        return 180.0D; 
      return 300.0D;
    } 
    double F = (60 * (M1 - M0) / (M2 - M0));
    if (rgb.r == M2) {
      if (rgb.g < rgb.b) {
        H = -F;
      } else {
        H = F;
      } 
    } else if (rgb.g == M2) {
      if (rgb.b < rgb.r) {
        H = -F;
      } else {
        H = F;
      } 
      H += 120.0D;
    } else {
      if (rgb.r < rgb.g) {
        H = -F;
      } else {
        H = F;
      } 
      H += 240.0D;
    } 
    if (H < 0.0D)
      H += 360.0D; 
    return H;
  }
  
  private static int maxColorValue(RGB rgb) {
    int max = rgb.r;
    if (rgb.g > max)
      max = rgb.g; 
    if (rgb.b > max)
      max = rgb.b; 
    return max;
  }
  
  private static int minColorValue(RGB rgb) {
    int min = rgb.r;
    if (rgb.g < min)
      min = rgb.g; 
    if (rgb.b < min)
      min = rgb.b; 
    return min;
  }
  
  private static int centerColorValue(RGB rgb) {
    if (rgb.r < rgb.g && rgb.r < rgb.b) {
      if (rgb.g < rgb.b)
        return rgb.g; 
      return rgb.b;
    } 
    if (rgb.g < rgb.r && rgb.g < rgb.b) {
      if (rgb.r < rgb.b)
        return rgb.r; 
      return rgb.b;
    } 
    if (rgb.r < rgb.g)
      return rgb.r; 
    return rgb.g;
  }
}
